/*----------------------------------------------------------------------------
/ M-800AD												2005 (C)LogicPack
/	05.12.12					000		M.Yamada
/		first release
/---------------------------------------------------------------------------*/
#include <machine.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "iodefine.h"
#include "sci.h"
#include "command.h"
#include "debug.h"
#include "mstring.h"
#include "m800ad.h"

#ifdef __cplusplus
extern "C" {
#endif
void abort(void);
#ifdef __cplusplus
}
#endif

/*-----------------------------------------------
/ *STATIC
/----------------------------------------------*/
/* message string */
const char prg[] = "M-800AD test V1.00";

/* prototype */
/*-----*/
static void disp_title(void);			/* ^Cg\ */
/*-----*/

void debug_out_int(const char *top, int data, const char *bottom);
void con_puts(const char *str);
void waitms(int ms);

bool cmd_help(const PARAM_INFO *param);
bool cmd_ad(const PARAM_INFO *param);
bool cmd_ads(const PARAM_INFO *param);
bool cmd_chad(const PARAM_INFO *param);
bool cmd_chads(const PARAM_INFO *param);
bool cmd_config(const PARAM_INFO *param);

static bool chk_adend(void);

/*-------------------------------------------------------------------
/ MAIN
/------------------------------------------------------------------*/
void main(void)
{
	set_imask_ccr(1);
	sci1_init();
	M800adInit();
	set_imask_ccr(0);

	disp_title();
	cmd_prompt("m800ad>");
	/* R}h̓o^ */
	cmd_entry("A",		1, cmd_ad);
	cmd_entry("B",		1, cmd_ads);
	cmd_entry("C",		1, cmd_chad);
	cmd_entry("D",		1, cmd_chads);
	cmd_entry("S",		1, cmd_config);
	cmd_entry("?",		1, cmd_help);
	cmd_entry("Help",	1, cmd_help);

	while(1){
		cmd_job();
	}
}

/*-------------------------------------------------------------------
/ abort
/------------------------------------------------------------------*/
void abort(void)
{
}

/*---------------------------------------------------------
/ ^Cg\
/--------------------------------------------------------*/
static void disp_title(void)
{
	static const char waku[] = 
		"-----------------------------------------------------------\r\n";
#if defined(RAM_EMULATE)
	waitms(2000);
#endif
	debug_out(waku);
	debug_out("--- ");	debug_out(prg);
#if defined(RAM_EMULATE)
	debug_out(" (ram emulation)\r\n");
#else
	debug_out("\r\n");
#endif
	debug_out(waku);
}

/*---------------------------------------------------------
/ R}hʃp[^́Zbg
/--------------------------------------------------------*/
/* Help */
/* ?    */
bool cmd_help(const PARAM_INFO *param)
{
	const char help[] = 
		" A <sel>       A/D start and read\r\n"
		" B <sel>       A/D start and read 1000data/msec\r\n"
		" C             A/D start and read 1`7ch\r\n"
		" D             A/D start and read 1`7ch 100data/msec\r\n"
		" S <CS> <bid>	Board ID Set\r\n"
		" Help or ?     disp help\r\n"
		"\r\n";
	con_out(help);
	return TRUE;
}

/* SET */
/* S <cs> <bid> */
bool cmd_config(const PARAM_INFO *param)
{
	int bid, cs;

	if( param->count != 2 ){ goto PDISP; }

	cs = str2dec(param->p[0]);		/* CPU /CSx */
	bid = str2dec(param->p[1]);		/* M800AD BORAD ID */

	if(!M800adConfig(cs,bid)){
PDISP:
		con_out("parameter error\r\n");
		return TRUE;
	}
	debug_out_int("M800AD access to /CS", cs, "\r\n");
	debug_out_int("M800AD BOARD ID=", bid, "\r\n\r\n");
	return TRUE;
}

/* AD */
/* A <ch> */
bool cmd_ad(const PARAM_INFO *param)
{
	int sel;

	sel = str2dec(param->p[0]);
	M800adChSel(sel);					/* chI */
	M800adStart(FALSE, FALSE);			/* ϊX^[g */
	if( chk_adend() ){
		word ad;
		ad = M800adGetData();			/* ADl擾 */
		debug_out_int("A/D ch", sel, " ");
		debug_outw3("data = 0x", ad, "\r\n\r\n");
	}else{
		debug_out("time out\r\n\r\n");
	}
	return TRUE;
}

#define BUFSIZE	1000
static word ad_buf[BUFSIZE];

/* ADA */
/* B <ch> */
bool cmd_ads(const PARAM_INFO *param)
{
	int sel;

	sel = str2dec(param->p[0]);
	M800adChSel(sel);						/* chI */
	M800adSetBuffer(ad_buf, BUFSIZE);		/* obt@ݒ */
	M800adStart(TRUE, FALSE);				/* ϊX^[g */

	if( chk_adend() ){
		long data;
		int i;
		debug_out_int("A/D ch", sel, "\r\n");
		for( i = 0; i < BUFSIZE; i++ ){
			data = ad_buf[i];
			data *= 20;
			data *= 1000;		/* mv */
			data /= 65536;
			data -= 10000;
			debug_out_int("", (int)data, "mV\r\n");
		}
	}else{
		debug_out("time out\r\n");
	}
	debug_out("\r\n\r\n");
	return TRUE;
}

/* AD(ch scan) */
/* C */
bool cmd_chad(const PARAM_INFO *param)
{
	/* ϊX^[g(ch scan mode) */
	M800adStart(FALSE, TRUE);
	if( chk_adend() ){
		word buf[8];
		int i;
		M800adGetDatas(buf);
		/* f[^\ */
		for( i = 0; i < 8; i++ ){
			debug_out_int("A/D ch", i, " ");
			debug_outw3("data = 0x", buf[i], "\r\n");
		}
	}else{
		debug_out("time out\r\n");
	}
	debug_out("\r\n");
	return TRUE;
}

/* ADA(ch scan) */
/* D */
bool cmd_chads(const PARAM_INFO *param)
{
	int i;
	/* e`lobt@ݒ */
	for( i = 0; i < 8; i++ ){
		M800adSetChBuffer(i, &ad_buf[100*i], 100);
	}
	M800adStart(TRUE, TRUE);		/* scan mode,AϊX^[g */
	if( chk_adend() ){
		int j;
		for( i = 0; i < 8; i++ ){
			debug_out_int("A/D ch", i, "");
			for( j  = 0; j < 100; j++ ){
				if( (j % 1) == 0 ){ debug_out("\r\n"); }
				debug_outw3("", ad_buf[(i*100)+j], " ");
			}
		}
	}else{
		debug_out("time out\r\n");
	}
	debug_out("\r\n\r\n");
	return TRUE;
}

/*---------------------------------------------------------
/ ADI`FbN
/--------------------------------------------------------*/
static bool chk_adend(void)
{
	long time;
	time = tim1ms;
	while( !M800adIsEnd() ){
		if( tim1ms - time >= 1000 ){ return FALSE; }	/* ^CAEg */
	}
	return TRUE;
}

/*-------------------------------------------------------------------
/ int 10i\
/------------------------------------------------------------------*/
void debug_out_int(const char *top, int data, const char *bottom)
{
	static char tmp[10];
	debug_out(top);
	debug_out(utoa(data, tmp));
	debug_out(bottom);
}

/*-------------------------------------------------------------------
/ R\[o
/------------------------------------------------------------------*/
void con_puts(const char *str)
{
	con_out(str);
	con_out("\r\n");
}
/*-------------------------------------------------------------------
/ EFCgimsecj
/------------------------------------------------------------------*/
void waitms(int ms)
{
	long time;
	time = tim1ms;
	while(tim1ms - time <= ms){;}
}
